use std::io::Read;
use std::io::Result;
use std::mem;
use std::usize;

pub struct Stream<T: Read> {
	reader: T,
	pub limit: usize,
}

impl<T> Stream<T> where T: Read {
	pub fn new(reader: T, limit: usize) -> Stream<T> {
		Stream { reader: reader, limit: limit }
	}

	pub fn limit(&self) -> usize {
		self.limit
	}

	pub fn consume(&mut self, amt: usize) {
		let mut cnt = amt;
		while cnt > 0 {
			self.read_u8().unwrap();
			cnt -= 1;
		}
	}

	// TODO
	pub fn read_exact(&mut self, buf: &mut [u8]) -> Result<()> {
		if self.limit < buf.len() {
			self.limit = 0;
			Ok(())
		} else {
			self.limit = self.limit.saturating_sub(buf.len());
			self.reader.read_exact(buf)
		}
	}

	pub fn read_until(&mut self, byte: u8, buf: &mut Vec<u8>) -> Result<usize> {
		let mut cnt: usize = 0;
		loop {
			let c = try!(self.read_u8());
			//self.limit.saturating_sub(1);			!!!
			cnt += 1;
			buf.push(c);
			if byte == c {
				break;
			}
		}

		Ok(cnt)
	}

	pub fn read_u32(&mut self) -> Result<u32> {
		let mut val: [u8; 4] = [0; 4];
		try!(self.read_exact(&mut val));
		unsafe {
			Ok(u32::from_be(mem::transmute::<[u8; 4], u32>(val)))
		}
	}

	pub fn read_u16(&mut self) -> Result<u16> {
		let mut val: [u8; 2] = [0; 2];
		try!(self.read_exact(&mut val));
		unsafe {
			Ok(u16::from_be(mem::transmute::<[u8; 2], u16>(val)))
		}
	}

	pub fn read_u8(&mut self) -> Result<u8> {
		let mut val: [u8; 1] = [0];
		try!(self.read_exact(&mut val));
		unsafe {
			Ok(mem::transmute::<[u8; 1], u8>(val))
		}
	}

	pub fn read_f32(&mut self) -> Result<f32> {
		unsafe {
			Ok(mem::transmute::<u32, f32>(try!(self.read_u32())))
		}
	}
}
